@@ -1,11 +1,12 @@
 package CPAN::Reporter;
 use strict;
 
-$CPAN::Reporter::VERSION = "0.43"; 
+$CPAN::Reporter::VERSION = "0.45"; 
 
 use Config;
 use Config::Tiny ();
 use CPAN ();
+use Fcntl qw/:flock :seek/;
 use File::Basename qw/basename/;
 use File::HomeDir ();
 use File::Path qw/mkpath rmtree/;
@@ -49,6 +50,21 @@ HERE
     }
 }
 
+#--------------------------------------------------------------------------#
+# Some platforms don't implement flock, so fake it if necessary
+#--------------------------------------------------------------------------#
+
+BEGIN {
+    eval {
+        my $fh = File::Temp->new();
+        flock $fh, LOCK_EX;
+    };
+    if ( $@ ) {
+        *CORE::GLOBAL::flock = sub () { 1 };
+    }
+}
+
+
 #--------------------------------------------------------------------------#
 # defaults and prompts
 #--------------------------------------------------------------------------#
@@ -356,8 +372,9 @@ END_BAD_DISTNAME
     $tr->grade( $result->{grade} );
     $tr->distribution( $result->{dist_name}  );
 
-    # Skip if duplicate, if requested
-    if ( _is_duplicate( $tr->subject ) ) {
+    # Skip if duplicate and not sending duplicates
+    my $is_duplicate = _is_duplicate( $tr->subject );
+    if ( $is_duplicate ) {
         if ( _prompt( $config, "send_duplicates", $tr->grade) =~ /^n/ ) {
             $CPAN::Frontend->mywarn(<< "DUPLICATE_REPORT");
 
@@ -403,7 +420,7 @@ DUPLICATE_REPORT
         $CPAN::Frontend->myprint( "Sending test report with '" . $tr->grade . 
               "' to " . join(q{, }, $tr->address, @cc) . "\n");
         if ( $tr->send( @cc ) ) {
-                _record_history( $tr->subject );
+                _record_history( $tr->subject ) if not $is_duplicate;
         }
         else {
             $CPAN::Frontend->mywarn( $tr->errstr. "\n");
@@ -426,9 +443,8 @@ sub _expand_report {
     my $result = shift;
 
     # Note: pretty_id is like "DAGOLDEN/CPAN-Reporter-0.40.tar.gz"
+    $result->{dist_name} = _format_distname( $result->{dist} );
     $result->{dist_basename} = basename($result->{dist}->pretty_id);
-    $result->{dist_name} = $result->{dist_basename};
-    $result->{dist_name} =~ s/(\.tar\.gz|\.tgz|\.zip)$//i;
     $result->{prereq_pm} = _prereq_report( $result->{dist} );
     $result->{env_vars} = _env_report();
     $result->{special_vars} = _special_vars_report();
@@ -486,6 +502,27 @@ sub _env_report {
     return $report;
 }
 
+#--------------------------------------------------------------------------#
+# _format_distname
+#--------------------------------------------------------------------------#
+
+sub _format_distname {
+    my $dist = shift;
+    my $basename = basename( $dist->pretty_id );
+    $basename =~ s/(\.tar\.gz|\.tgz|\.zip)$//i;
+    return $basename;
+}
+
+#--------------------------------------------------------------------------#
+# _format_history -- append perl version to subject
+#--------------------------------------------------------------------------#
+
+sub _format_history {
+    my $line = shift(@_) . " $]"; # append perl version to subject
+    $line .= " patch $Config{perl_patchlevel}" if $Config{perl_patchlevel};
+    return $line . "\n";
+}
+
 #--------------------------------------------------------------------------#
 # _get_config_dir
 #--------------------------------------------------------------------------#
@@ -582,6 +619,10 @@ sub _grade_report {
             $grade = 'fail';
             $msg = 'Tests had no output';
         }
+        elsif ( $output->[$i] =~ m{FAILED--Further testing stopped}ms ) {
+            $grade = 'fail';
+            $msg = 'Bailed out of tests';
+        }
         elsif ( $output->[$i] =~ m{^Failed }ms ) {  # must be lowercase
             $grade = 'fail';
             $msg = "Distribution had failing tests";
@@ -619,10 +660,10 @@ sub _grade_report {
         }
     }
 
-    # Downgrade failure if we can determine a cause of the failure
-    # that should be reported as 'na'
+    # Downgrade failure/unknown grade if we can determine a cause
+    # that should be reported as 'na'.
 
-    if ( $grade eq 'fail' ) {
+    if ( $grade eq 'fail' || $grade eq 'unknown' ) {
         # check for perl version prerequisite or outright failure
         if ( $result->{prereq_pm} =~ m{^\s+!\s+perl\s}ims ) {
             $grade = 'na';
@@ -667,11 +708,15 @@ sub _has_tests {
 #--------------------------------------------------------------------------#
 
 sub _is_duplicate {
-    my $subject = shift(@_) . " $]"; # append perl version to subject
+    my $subject = _format_history( shift );
     my $history = _open_history_file('<') or return;
-    my %seen = map { chomp; ($_, 1); } <$history>;
+    my $found = 0;
+    flock $history, LOCK_SH;
+    while ( defined (my $line = <$history>) ) {
+        $found++, last if $line eq $subject
+    }
     $history->close;
-    return exists $seen{$subject};
+    return $found;
 }
 
 #--------------------------------------------------------------------------#
@@ -880,9 +925,14 @@ sub _prompt {
 #--------------------------------------------------------------------------#
 
 sub _record_history {
-    my $subject = shift(@_) . " $]"; # append perl version to subject
+    my $subject = _format_history( shift );
     my $history = _open_history_file('>>') or return;
-    $history->print( $subject, "\n" );
+
+    flock( $history, LOCK_EX );
+    seek( $history, 0, SEEK_END );
+    $history->print( $subject );
+    flock( $history, LOCK_UN );
+    
     $history->close;
     return;
 }
