@@ -2,6 +2,17 @@ package t::MockCPANDist;
 use strict;
 BEGIN { if ( not $] < 5.006 ) { require warnings; warnings->import } }
 
+#--------------------------------------------------------------------------#
+
+my $simulate_bad_author = 0;
+
+sub import {
+    my $class = shift;
+    $simulate_bad_author = grep { $_ eq 'bad_author' } @_;
+}
+
+#--------------------------------------------------------------------------#
+
 my %spec = (
     prereq_pm => 'HASH',
     pretty_id => q{},
@@ -26,10 +37,12 @@ sub new {
     bless \%args, $class;
 }
 
-sub author { return shift } # cheat and let the mock handle it all
+# cheat on author() and let the mock handle it all unless we want it to fail
+sub author { return $simulate_bad_author ? undef : shift } 
+
 sub prereq_pm { return shift->{prereq_pm} }
 sub pretty_id { return shift->{pretty_id} }
-sub id { return shift->{id} }
+sub id { return shift->{author_id} }
 sub fullname { return shift->{author_fullname} }
 
 1;
